@@ -7,7 +7,6 @@ import testbase
 from tables import *
 import tables
 
-
 class HistoryTest(AssertMixin):
     def setUpAll(self):
         db.echo = False
@@ -76,7 +75,6 @@ class SessionTest(AssertMixin):
     def setUpAll(self):
         db.echo = False
         users.create()
-        tables.user_data()
         db.echo = testbase.echo
     def tearDownAll(self):
         db.echo = False
@@ -85,6 +83,10 @@ class SessionTest(AssertMixin):
     def setUp(self):
         objectstore.get_session().clear()
         clear_mappers()
+        tables.user_data()
+        #db.echo = "debug"
+    def tearDown(self):
+        tables.delete_user_data()
         
     def test_nested_begin_commit(self):
         """test nested session.begin/commit"""
@@ -97,37 +99,82 @@ class SessionTest(AssertMixin):
         self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
         self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
         s = objectstore.get_session()
-        s.begin()
-        s.begin()
+        trans = s.begin()
+        trans2 = s.begin()
         m.get(7).user_name = name1
-        s.begin()
+        trans3 = s.begin()
         m.get(8).user_name = name2
-        s.commit()
+        trans3.commit()
+        s.commit() # should do nothing
         self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
         self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
-        s.commit()
+        trans2.commit()
+        s.commit()  # should do nothing
         self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
         self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
-        s.commit()
+        trans.commit()
         self.assert_(name_of(7) == name1, msg="user_name should be %s" % name1)
         self.assert_(name_of(8) == name2, msg="user_name should be %s" % name2)
 
+    def test_nested_rollback(self):
+        class User(object):pass
+        m = mapper(User, users)
+        def name_of(id):
+            return users.select(users.c.user_id == id).execute().fetchone().user_name
+        name1 = "Oliver Twist"
+        name2 = 'Mr. Bumble'
+        self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
+        self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
+        s = objectstore.get_session()
+        trans = s.begin()
+        trans2 = s.begin()
+        m.get(7).user_name = name1
+        trans3 = s.begin()
+        m.get(8).user_name = name2
+        trans3.rollback()
+        self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
+        self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
+        trans2.commit()
+        self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
+        self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
+        trans.commit()
+        self.assert_(name_of(7) != name1, msg="user_name should not be %s" % name1)
+        self.assert_(name_of(8) != name2, msg="user_name should not be %s" % name2)
 
 class PKTest(AssertMixin):
     def setUpAll(self):
         db.echo = False
-        self.table = Table(
+        global table
+        global table2
+        global table3
+        table = Table(
             'multi', db, 
             Column('multi_id', Integer, Sequence("multi_id_seq", optional=True), primary_key=True),
             Column('multi_rev', Integer, primary_key=True),
             Column('name', String(50), nullable=False),
             Column('value', String(100))
         )
-        self.table.create()
+        
+        table2 = Table('multi2', db,
+            Column('pk_col_1', String(30), primary_key=True),
+            Column('pk_col_2', String(30), primary_key=True),
+            Column('data', String(30), )
+            )
+        table3 = Table('multi3', db,
+            Column('pri_code', String(30), key='primary', primary_key=True),
+            Column('sec_code', String(30), key='secondary', primary_key=True),
+            Column('date_assigned', Date, key='assigned', primary_key=True),
+            Column('data', String(30), )
+            )
+        table.create()
+        table2.create()
+        table3.create()
         db.echo = testbase.echo
     def tearDownAll(self):
         db.echo = False
-        self.table.drop()
+        table.drop()
+        table2.drop()
+        table3.drop()
         db.echo = testbase.echo
     def setUp(self):
         objectstore.clear()
@@ -135,7 +182,7 @@ class PKTest(AssertMixin):
     def testprimarykey(self):
         class Entry(object):
             pass
-        Entry.mapper = mapper(Entry, self.table)
+        Entry.mapper = mapper(Entry, table)
         e = Entry()
         e.name = 'entry1'
         e.value = 'this is entry 1'
@@ -144,7 +191,27 @@ class PKTest(AssertMixin):
         objectstore.clear()
         e2 = Entry.mapper.get(e.multi_id, 2)
         self.assert_(e is not e2 and e._instance_key == e2._instance_key)
-
+    def testmanualpk(self):
+        class Entry(object):
+            pass
+        Entry.mapper = mapper(Entry, table2)
+        e = Entry()
+        e.pk_col_1 = 'pk1'
+        e.pk_col_2 = 'pk1_related'
+        e.data = 'im the data'
+        objectstore.commit()
+    def testkeypks(self):
+        import datetime
+        class Entity(object):
+            pass
+        Entity.mapper = mapper(Entity, table3)
+        e = Entity()
+        e.primary = 'pk1'
+        e.secondary = 'pk2'
+        e.assigned = datetime.date.today()
+        e.data = 'some more data'
+        objectstore.commit()
+        
 class DefaultTest(AssertMixin):
     def setUpAll(self):
         #db.echo = 'debug'
